GRAPH_ENDPOINT = 'https://graph.facebook.com/v10.0/'
INSIGHTS_CSV = 'insta_insights.csv'
HTML_FILE = 'index.html'
CSS_FILE = 'style.css'
HTML_TEMPLATE = 'index.html.template'

ID_COL = 'id'
IMPRESSIONS_COL = 'impressions'
ENGAGEMENT_COL = 'engagement'
REACH_COL = 'reach'
TIMESTAMP_COL = 'timestamp'
HOUR_COL = 'hour'

DEFAULT_COLUMNS = [ID_COL, IMPRESSIONS_COL, ENGAGEMENT_COL, REACH_COL, TIMESTAMP_COL, HOUR_COL]
METRICS_COLUMNS = [IMPRESSIONS_COL, ENGAGEMENT_COL, REACH_COL]



import constant
from commons import readcsv
from commons import remove_element_list
import insights.htmlutils as htmlutils


import matplotlib.pyplot as plt


def get_best_tags(show_graphs=False, max_indices=4):
    df = readcsv()
    for metric in constant.METRICS_COLUMNS:
        _get_best_tag_based_on(metric, df, max_indices=max_indices, show_graphs=show_graphs)


def _get_best_tag_based_on(column_name, initial_df, max_indices, show_graphs=False):
    tag_avg_impressions = dict()
    df = initial_df.copy(deep=True)
    columns_without_impressions = remove_element_list(constant.DEFAULT_COLUMNS, column_name)
    df = df.drop(columns_without_impressions, axis=1)
    tags_columns = remove_element_list(df.columns.values, column_name)
    for tag in tags_columns:
        tag_avg_impressions.update({tag: (df[tag]*df[column_name]).sum()/(df[tag].sum())})
    sorted_avg = dict(sorted(tag_avg_impressions.items(), key=lambda item: item[1]))

    table = htmlutils.Table('Tags based on ' + column_name)
    table.add_header_row(['Best tags', 'Metric values', 'Worst tags', 'Metric values'])

    print(f'Best  tags based on {column_name}           : {list(sorted_avg.keys())[-max_indices:][::-1]}')
    print(f'Their corresponding {column_name} values are: {list(sorted_avg.values())[-max_indices:][::-1]}')
    print()

    table.add_column_data(list(sorted_avg.keys())[-max_indices:][::-1])
    table.add_column_data(list(sorted_avg.values())[-max_indices:][::-1])

    print(f'Worst tags based on {column_name}           : {list(sorted_avg.keys())[:max_indices]}')
    print(f'Their corresponding {column_name} values are: {list(sorted_avg.values())[:max_indices]}')
    print()

    table.add_column_data(list(sorted_avg.keys())[:max_indices])
    table.add_column_data(list(sorted_avg.values())[:max_indices])

    table.write_html()

    if show_graphs:
        plt.bar(*zip(*sorted_avg.items()))
        plt.show()
    return list(sorted_avg.keys())[-1], list(sorted_avg.keys())[0]



import constant
import os
import pkgutil
import pkg_resources
import shutil

def get_html_file():
    return os.getcwd() + "/index.html"


def get_css_file():
    return os.getcwd() + "/style.css"


def create_html():
    try:
        shutil.copy(pkg_resources.resource_filename(__name__, "data/" + constant.HTML_TEMPLATE), get_html_file())
        shutil.copy(pkg_resources.resource_filename(__name__, "data/" + constant.CSS_FILE), get_css_file())
    except Exception as ex:
        print("Warning: Exception while copying HTML or CSS files." + str(ex))
        pass


def end_html():
    append_html('</div></body>' + "\n" + '</html>')
    print('HTML file is generated at ', get_html_file())


def append_html(html_data):
    with open(get_html_file(), 'a+') as htmlFile:
        htmlFile.write(html_data)


class Heading:
    def __init__(self, text):
        append_html('<h3>' + text + '</h3>')


class Table:
    def __init__(self, caption):
        self.list_column_lists = []
        self.__header_columns = []
        self.__caption = caption
        self.html = '<table>'

    def add_header_row(self, header_columns_names):
        self.__header_columns = header_columns_names

    def add_column_data(self, column_data):
        self.list_column_lists.append(column_data)

    def write_html(self):
        self.html += '<caption>' + self.__caption + '</caption>' + "\n"
        self.__add_row(self.__header_columns, header=True)
        num_rows = len(self.list_column_lists[0])  # All columns have same length
        for index in range(num_rows):
            row = []
            for data_item_list in self.list_column_lists:
                row.append(data_item_list[index])
            self.__add_row(row)
        self.html += '</table>' + '</br>' + '</br>' + "\n"
        append_html(self.html)

    def __add_row(self, row, header=False):
        row_html = '    <tr>'
        table_data_start_tag = '<th>' if header else '<td>'
        table_data_end_tag = '</th>' if header else '</td>'
        for item in row:
            row_html += table_data_start_tag + str(item) + table_data_end_tag
        row_html += "\n"
        self.html += row_html



from commons import readcsv
import numpy as np
import constant
import insights.htmlutils as htmlutils


def get_timing_insights(max_indices=4):
    df = readcsv()
    for metric in constant.METRICS_COLUMNS:
        _get_best_timing_based_on(metric, df, max_indices=max_indices)


def _get_best_timing_based_on(column_name, df, max_indices=3):
    # corresponds to total metric value of all the posts for that particular hour
    metrics_total = np.full([24], 0, dtype=int)

    # corresponds to number of times that a post has been made in that hour
    metrics_counter = np.full([24], 0, dtype=int)

    table = htmlutils.Table('Timings based on ' + column_name)

    for index, row in df.iterrows():
        metrics_total[row[constant.HOUR_COL]] += row[column_name]
        metrics_counter[row[constant.HOUR_COL]] += 1
    metrics_avg = np.divide(metrics_total, metrics_counter, out=np.zeros(metrics_total.shape, dtype=float), where=metrics_counter!=0)

    # Find a total of max_indices maximums
    idx = np.argpartition(metrics_avg, -max_indices)[-max_indices:]
    idx = idx[np.argsort(metrics_avg[idx])][::-1]

    np.set_printoptions(formatter={'float_kind': "{:.1f}".format})
    print(f'Top {max_indices} hours to post for maximum {column_name} are {idx}')
    print(f'Average {column_name}s at these hours are {metrics_avg[idx]} respectively.')
    print()

    table.add_header_row(['Best hours', 'Average metric values'])
    table.add_column_data(idx)
    table.add_column_data(metrics_avg[idx])
    table.write_html()


from .restmethods import get
import constant
import re
import csv
from datetime import datetime


class Instapost:
    def __init__(self, igmedia_id, impressions, engagement, reach, timestamp, caption):
        self.id = igmedia_id
        self.impressions = impressions
        self.engagement = engagement
        self.reach = reach
        self.timestamp = timestamp
        self._caption = caption
        self.tags = self._get_hash_tags()
        self.hour = self._get_hour()

    def print_post(self):
        print(self.id, end=' ')
        print(*self.tags, sep=' ')

    def _get_hash_tags(self):
        return [tag[1:] for tag in re.findall("[#]\w+", self._caption)]

    def _get_hour(self):
        # Sample date format is 2020-12-20T11:29:31+0000
        return datetime.strptime(self.timestamp, "%Y-%m-%dT%H:%M:%S%z").hour


def get_insights(token, page_id):
    if page_id is None or token is None:
        raise Exception("Token or page_id is none.")
    posts_response = get([page_id, 'media'], token)
    posts = []
    unique_tags = set()
    for post_dict in posts_response['data']:
        post_id = post_dict['id']
        insights_resp = get([post_id, 'insights'], token, query_params={'metric': 'impressions,engagement,reach'})
        metadata_resp = get([post_id], token, query_params={'fields': 'caption,timestamp'})
        insights_data = insights_resp['data']

        current_post = Instapost(post_id,
                                 _getvalue(insights_data[0]),
                                 _getvalue(insights_data[1]),
                                 _getvalue(insights_data[2]),
                                 metadata_resp['timestamp'],
                                 metadata_resp['caption'])
        posts.append(current_post)
        unique_tags.update(current_post.tags)
        unique_tags_list = list(unique_tags)
        headers_row = constant.DEFAULT_COLUMNS + unique_tags_list

        with open(constant.INSIGHTS_CSV, 'w') as csvfile:
            csvwriter = csv.writer(csvfile)
            csvwriter.writerow(headers_row)
            for post in posts:
                metadata = [post.id, post.impressions, post.engagement, post.reach, post.timestamp, post.hour]
                tags_list = list(map(lambda tag: 1 if tag in post.tags else 0, unique_tags_list))
                total_row = metadata + tags_list
                csvwriter.writerow(total_row)


def _getvalue(insights_data_object):
    return insights_data_object['values'][0]['value']




import constant
import pandas


def readcsv():
    return pandas.read_csv(constant.INSIGHTS_CSV)


def remove_element_list(input_list, to_be_removed):
    return [elem for elem in input_list if elem != to_be_removed]


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="instagram-insights",
    version="0.0.dev5",
    author="Pardhu Madipalli",
    author_email="pardhu.madipalli@gmail.com",
    description="Get insights about best times to post, and the best tags based on likes, views and impressions.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/PardhuMadipalli/instagram-insights",
    project_urls={
        "Bug Tracker": "https://github.com/PardhuMadipalli/instagram-insights/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    package_data={'insights': ['data/*']},
    data_files=[('.', ['screenshot.png', 'README.md'])],
    py_modules = ['constant', 'setup', 'get_insta_insights', 'commons'],
    python_requires=">=3.6",
    install_requires=['numpy', 'pandas'],
    entry_points={"console_scripts": ["insta-insights=get_insta_insights:main"]},
)



#!/usr/bin/env python3

from getopt import getopt
import sys
import os

from instagram import instagram_data
from insights import hashtags
from insights import timings
from insights import htmlutils

options = "mh"
long_options = ["machine-learning", "help", "page-id=", "token="]


def usage():
    print(f"""usage:

insta-insights --page-id=<page-id> --token=<token>

Entering page ID and token everytime when insta-insights is invoked can be avoided by setting them as env variables.
    
Environment variables supported:
FB_TOKEN
FB_PAGE_ID

Read more at: https://github.com/PardhuMadipalli/instagram-insights#quickstart
""")


def main():
    token = None
    page_id = None
    try:
        """ Parse Arguments """
        arguments, values = getopt(sys.argv[1:], options, long_options)
        for currentArgument, currentValue in arguments:
            if currentArgument in ["-h", "--help"]:
                usage()
                sys.exit(0)
            if currentArgument in ["-m", "--machine-learning"]:
                raise RuntimeError('Machine learning is not supported yet. Use -h for help.')
            if currentArgument == "--page-id":
                page_id = currentValue
            if currentArgument == "--token":
                token = currentValue
        if token is None:
            token = os.environ.get("FB_TOKEN")
        if page_id is None:
            page_id = os.environ.get("FB_PAGE_ID")

        """ Main function starts here"""

        instagram_data.get_insights(token, page_id)

        htmlutils.create_html()

        timings.get_timing_insights(max_indices=3)
        hashtags.get_best_tags(max_indices=5)

        htmlutils.end_html()
    except Exception as ex:
        usage()
        raise ex


if __name__ == "__main__":
    main()



import constant
import requests
import json


def get(path_params, token, query_params=None):
    if query_params is None:
        query_params = {}
    resp = requests.get(url=_geturl(path_params), params={**_get_token_query(token), **query_params})
    if resp.status_code not in [200, 201]:
        raise Exception('Incorrect response code.', resp.text)
    return json.loads(requests.get(url=_geturl(path_params), params={**_get_token_query(token), **query_params}).text)


def _geturl(path_params):
    constructed_path = ''
    for param in path_params:
        constructed_path += param + '/'
    return constant.GRAPH_ENDPOINT + constructed_path


def _get_token_query(token):
    return {'access_token': token}
